@@ -1,14 +1,14 @@
 use anyhow::{anyhow, ensure, Result};
-use cargo_metadata::{Metadata, MetadataCommand, Package, Target};
+use cargo_metadata::{Metadata, Package, Target};
 use compiletest_rs::{self as compiletest, common::Mode as TestMode};
 use dylint_internal::{
-    env::{self, var},
-    Command,
+    cargo::{self, current_metadata, root_package},
+    env,
 };
 use lazy_static::lazy_static;
 use regex::Regex;
 use std::{
-    env::set_var,
+    env::{consts, set_var},
     fs::{copy, read_dir, remove_file},
     io::BufRead,
     path::Path,
@@ -25,8 +25,8 @@ pub fn ui_test(name: &str, src_base: &Path) {
 pub fn ui_test_example(name: &str, example: &str) {
     let driver = initialize(name).unwrap();
 
-    let metadata = metadata().unwrap();
-    let package = package(&metadata).unwrap();
+    let metadata = current_metadata().unwrap();
+    let package = root_package(&metadata).unwrap();
     let target = example_target(&package, example).unwrap();
 
     run_example_test(&driver, &metadata, &package, &target).unwrap();
@@ -35,8 +35,8 @@ pub fn ui_test_example(name: &str, example: &str) {
 pub fn ui_test_examples(name: &str) {
     let driver = initialize(name).unwrap();
 
-    let metadata = metadata().unwrap();
-    let package = package(&metadata).unwrap();
+    let metadata = current_metadata().unwrap();
+    let package = root_package(&metadata).unwrap();
     let targets = example_targets(&package).unwrap();
 
     for target in targets {
@@ -53,11 +53,9 @@ fn initialize(name: &str) -> Result<PathBuf> {
     dylint_internal::build().success()?;
 
     // smoelius: `DYLINT_LIBRARY_PATH` must be set before `dylint_libs` is called.
-    let manifest_dir = var(env::CARGO_MANIFEST_DIR)?;
-    set_var(
-        env::DYLINT_LIBRARY_PATH,
-        Path::new(&manifest_dir).join("target").join("debug"),
-    );
+    let metadata = current_metadata().unwrap();
+    let dylint_library_path = metadata.target_directory.join("debug");
+    set_var(env::DYLINT_LIBRARY_PATH, dylint_library_path);
 
     let dylint_libs = dylint_libs(name)?;
     let driver = dylint::driver_builder::get(&dylint::Dylint::default(), env!("RUSTUP_TOOLCHAIN"))?;
@@ -76,25 +74,6 @@ pub fn dylint_libs(name: &str) -> Result<String> {
     serde_json::to_string(&paths).map_err(Into::into)
 }
 
-fn metadata() -> Result<Metadata> {
-    let manifest_dir = var(env::CARGO_MANIFEST_DIR)?;
-    let manifest_path = Path::new(&manifest_dir).join("Cargo.toml");
-    MetadataCommand::new()
-        .manifest_path(manifest_path)
-        .no_deps()
-        .exec()
-        .map_err(Into::into)
-}
-
-fn package(metadata: &Metadata) -> Result<Package> {
-    ensure!(metadata.packages.len() <= 1, "Found multiple packages");
-    metadata
-        .packages
-        .first()
-        .cloned()
-        .ok_or_else(|| anyhow!("Found no packages"))
-}
-
 fn example_target(package: &Package, example: &str) -> Result<Target> {
     package
         .targets
@@ -144,17 +123,17 @@ fn linking_flags(metadata: &Metadata, package: &Package, target: &Target) -> Res
 
     let mut linking_flags = Vec::new();
 
-    let mut iter = rustc_flags.iter();
+    let mut iter = rustc_flags.into_iter();
     while let Some(flag) = iter.next() {
         if flag.starts_with("--edition=") {
             linking_flags.push(flag);
         } else if flag == "--extern" || flag == "-L" {
-            let arg = next(flag, &mut iter)?;
-            linking_flags.extend_from_slice(&[flag, arg])
+            let arg = next(&flag, &mut iter)?;
+            linking_flags.extend(vec![flag, arg.trim_matches('\'').to_owned()]);
         }
     }
 
-    Ok(linking_flags.into_iter().cloned().collect())
+    Ok(linking_flags)
 }
 
 // smoelius: We need to recover the `rustc` flags used to build a target. I can see three options:
@@ -181,6 +160,7 @@ fn rustc_flags(metadata: &Metadata, package: &Package, target: &Target) -> Resul
     // smoelius: Force rebuilding of the example by removing it. This is kind of messy. The example
     // is a shared resource that may be needed by multiple tests. For now, I lock a mutex while the
     // example is removed and put back.
+    // smoelius: Should we use a temporary target directory here?
     let output = {
         let _guard = LOCK
             .lock()
@@ -188,9 +168,9 @@ fn rustc_flags(metadata: &Metadata, package: &Package, target: &Target) -> Resul
 
         remove_example(metadata, package, target)?;
 
-        Command::new("cargo")
+        cargo::build()
+            .envs(vec![(env::CARGO_TERM_COLOR, "never")])
             .args(&[
-                "build",
                 "--manifest-path",
                 &package.manifest_path.to_string(),
                 "--example",
@@ -242,7 +222,9 @@ fn remove_example(metadata: &Metadata, _package: &Package, target: &Target) -> R
 
         if let Some(file_name) = path.file_name() {
             let s = file_name.to_string_lossy();
-            if s == target.name || s.starts_with(&(target.name.clone() + "-")) {
+            if s == target.name.clone() + consts::EXE_SUFFIX
+                || s.starts_with(&(target.name.clone() + "-"))
+            {
                 remove_file(path)?;
             }
         }
